@@ -37,7 +37,7 @@ def encrypt(password, data):
     counter = Counter.new(HALF_BLOCK, prefix=salt[:HALF_BLOCK//8])
     cipher = AES.new(cipher_key, AES.MODE_CTR, counter=counter)
     encrypted = cipher.encrypt(data)
-    hmac = HMAC.new(hmac_key, HEADER + salt + encrypted, HASH).digest()
+    hmac = _hmac(hmac_key, HEADER + salt + encrypted)
     return HEADER + salt + encrypted + hmac
 
 
@@ -61,8 +61,8 @@ def decrypt(password, data):
     salt = raw[:SALT_LEN//8]
     hmac_key, cipher_key = _expand_keys(password, salt)
     hmac = raw[-HASH.digest_size:]
-    hmac2 = HMAC.new(hmac_key, data[:-HASH.digest_size], HASH).digest()
-    _assert_hmac(hmac, hmac2)
+    hmac2 = _hmac(hmac_key, data[:-HASH.digest_size])
+    _assert_hmac(hmac_key, hmac, hmac2)
     counter = Counter.new(HALF_BLOCK, prefix=salt[:HALF_BLOCK//8])
     cipher = AES.new(cipher_key, AES.MODE_CTR, counter=counter)
     return cipher.decrypt(raw[SALT_LEN//8:-HASH.digest_size])
@@ -88,17 +88,17 @@ def _assert_decrypt_length(data):
         raise DecryptionException('Missing data.')
     
 def _assert_header_sc(data):
-    if len(data) < 2 or data[:2] != HEADER[:2]:
+    if len(data) >= 2 and data[:2] != HEADER[:2]:
         raise DecryptionException('Data passed to decrypt were not generated by simple-crypt (bad header).')
 
 def _assert_header_version(data):
-    if len(data) < len(HEADER) or data[:len(HEADER)] != HEADER:
+    if len(data) >= len(HEADER) and data[:len(HEADER)] != HEADER:
         raise DecryptionException('The data appear to be encrypted with a more recent version of simple-crypt (bad header). ' +
         'Please update the library and try again.')
 
-def _assert_hmac(hmac, hmac2):
+def _assert_hmac(key, hmac, hmac2):
     # https://www.isecpartners.com/news-events/news/2011/february/double-hmac-verification.aspx
-    if _hash(hmac) != _hash(hmac2):
+    if _hmac(key, hmac) != _hmac(key, hmac2):
         raise DecryptionException('Bad password or corrupt / modified data.')
 
 def _expand_keys(password, salt):
@@ -113,8 +113,8 @@ def _expand_keys(password, salt):
 def _random_bytes(n):
     return bytes(getrandbits(8) for _ in range(n))
 
-def _hash(data):
-    return HASH.new(data=data).digest()
+def _hmac(key, data):
+    return HMAC.new(key, data, HASH).digest()
 
 def _str_to_bytes(data):
     try: return data.encode('utf8')
